@@ -28,6 +28,11 @@ GType arv_acquisition_mode_get_type	(void) G_GNUC_CONST;
 #define ARV_TYPE_PIXEL_FORMAT	(arv_pixel_format_get_type())
 GType arv_pixel_format_get_type	(void) G_GNUC_CONST;
 
+/* Enumerations from "arvdebug.h" */
+
+#define ARV_TYPE_DEBUG_LEVEL	(arv_debug_level_get_type())
+GType arv_debug_level_get_type	(void) G_GNUC_CONST;
+
 /* Enumerations from "arvgcregister.h" */
 
 #define ARV_TYPE_GC_SIGN	(arv_gc_sign_get_type())
@@ -67,6 +72,9 @@ GType arv_gvcp_command_get_type	(void) G_GNUC_CONST;
 #define ARV_TYPE_GVSP_PACKET_TYPE	(arv_gvsp_packet_type_get_type())
 GType arv_gvsp_packet_type_get_type	(void) G_GNUC_CONST;
 
+#define ARV_TYPE_GVSP_CONTENT_TYPE	(arv_gvsp_content_type_get_type())
+GType arv_gvsp_content_type_get_type	(void) G_GNUC_CONST;
+
 G_END_DECLS
 
 #endif /* ARV_ENUM_TYPES_H */
