@@ -30,6 +30,7 @@
 
 #include <arvgcnode.h>
 #include <arvgc.h>
+#include <arvmisc.h>
 #include <arvdebug.h>
 #include <string.h>
 
@@ -43,6 +44,9 @@ struct _ArvGcNodePrivate {
 	char *description;
 	char *display_name;
 
+	GValue is_implemented;
+	GValue is_available;
+
 	unsigned int n_childs;
 	GSList *childs;
 
@@ -81,6 +85,15 @@ arv_gc_node_get_display_name (ArvGcNode *node)
 	return node->priv->display_name;
 }
 
+gboolean
+arv_gc_node_is_available (ArvGcNode *gc_node)
+{
+	g_return_val_if_fail (ARV_IS_GC_NODE (gc_node), FALSE);
+
+	return arv_gc_get_int64_from_value (gc_node->priv->genicam, &gc_node->priv->is_implemented) != 0 &&
+		arv_gc_get_int64_from_value (gc_node->priv->genicam, &gc_node->priv->is_available) != 0;
+}
+
 static void
 _set_attribute (ArvGcNode *node, const char *name, const char *value)
 {
@@ -116,6 +129,10 @@ _add_element (ArvGcNode *node, const char *name, const char *content, const char
 	} else if (strcmp (name, "DisplayName") == 0) {
 		g_free (node->priv->display_name);
 		node->priv->display_name = g_strdup (content);
+	} else if (strcmp (name, "pIsImplemented") == 0) {
+		arv_force_g_value_to_string (&node->priv->is_implemented, content);
+	} else if (strcmp (name, "pIsAvailable") == 0) {
+		arv_force_g_value_to_string (&node->priv->is_available, content);
 	}
 }
 
@@ -182,6 +199,29 @@ arv_gc_node_new (void)
 	return node;
 }
 
+/**
+ * arv_gc_node_get_node_name:
+ * @gc_node: a #ArvGcNode
+ *
+ * Retrieves the Genicam name of the given node ("Integer", "IntReg", ...).
+ *
+ * Returns: The node name, %NULL on error.
+ */
+
+const char *
+arv_gc_node_get_node_name (ArvGcNode *gc_node)
+{
+	ArvGcNodeClass *gc_node_class;
+
+	g_return_val_if_fail (ARV_IS_GC_NODE (gc_node), FALSE);
+
+	gc_node_class = ARV_GC_NODE_GET_CLASS (gc_node);
+	if (gc_node_class->get_node_name != NULL)
+	       return gc_node_class->get_node_name (gc_node);
+
+	return NULL;
+}
+
 void
 arv_gc_node_set_genicam	(ArvGcNode *node, ArvGc *genicam)
 {
@@ -221,6 +261,52 @@ arv_gc_node_get_value_type (ArvGcNode *node)
 	return 0;
 }
 
+/**
+ * arv_gc_node_set_value_from_string:
+ * @gc_node: a #ArvGcNode
+ * @string: new node value, as string
+ *
+ * Set the node value using a string representation of the value. May not be applicable to every node type, but safe.
+ */
+
+void
+arv_gc_node_set_value_from_string (ArvGcNode *gc_node, const char *string)
+{
+	ArvGcNodeClass *node_class;
+
+	g_return_if_fail (ARV_IS_GC_NODE (gc_node));
+	g_return_if_fail (string != NULL);
+
+	node_class = ARV_GC_NODE_GET_CLASS (gc_node);
+	if (node_class->set_value_from_string != NULL)
+		node_class->set_value_from_string (gc_node, string);
+}
+
+/**
+ * arv_gc_node_get_value_as_string:
+ * @gc_node: a #ArvGcNode
+ *
+ * Retrieve the node value a string.
+ *
+ * <warning><para>Please not the string content is still owned by the @node object, which means the returned pointer may not be still valid after a new call to this function.</para></warning>
+ *
+ * Returns: (transfer none): a string representation of the node value, %NULL if not applicable.
+ */
+
+const char *
+arv_gc_node_get_value_as_string (ArvGcNode *gc_node)
+{
+	ArvGcNodeClass *node_class;
+
+	g_return_val_if_fail (ARV_IS_GC_NODE (gc_node), NULL);
+
+	node_class = ARV_GC_NODE_GET_CLASS (gc_node);
+	if (node_class->get_value_as_string != NULL)
+		return node_class->get_value_as_string (gc_node);
+
+	return NULL;
+}
+
 void
 arv_gc_node_inc_modification_count (ArvGcNode *gc_node)
 {
@@ -249,6 +335,11 @@ arv_gc_node_init (ArvGcNode *gc_node)
 	gc_node->priv->childs = NULL;
 	gc_node->priv->n_childs = 0;
 
+	g_value_init (&gc_node->priv->is_implemented, G_TYPE_INT64);
+	g_value_set_int64 (&gc_node->priv->is_implemented, 1);
+	g_value_init (&gc_node->priv->is_available, G_TYPE_INT64);
+	g_value_set_int64 (&gc_node->priv->is_available, 1);
+
 	gc_node->priv->modification_count = 0;
 }
 
@@ -268,6 +359,9 @@ arv_gc_node_finalize (GObject *object)
 	g_free (node->priv->description);
 	g_free (node->priv->display_name);
 
+	g_value_unset (&node->priv->is_implemented);
+	g_value_unset (&node->priv->is_available);
+
 	parent_class->finalize (object);
 }
 
