@@ -30,7 +30,7 @@
 #include <arvgvsp.h>
 #include <arvgvcp.h>
 #include <arvdebug.h>
-#include <arvtools.h>
+#include <arvmisc.h>
 #include <arvenumtypes.h>
 #include <string.h>
 #include <sys/socket.h>
@@ -41,6 +41,8 @@
 #define ARV_GV_STREAM_PACKET_TIMEOUT_US_DEFAULT		40000
 #define ARV_GV_STREAM_FRAME_RETENTION_US_DEFAULT	200000
 
+#define ARV_GV_STREAM_DISCARD_LATE_FRAME_THRESHOLD	100
+
 enum {
 	ARV_GV_STREAM_PROPERTY_0,
 	ARV_GV_STREAM_PROPERTY_SOCKET_BUFFER,
@@ -67,6 +69,8 @@ typedef struct {
 	guint64 first_packet_time_us;
 	guint64 last_packet_time_us;
 
+	gboolean error_packet_received;
+
 	guint n_packets;
 	ArvGvStreamPacketData *packet_data;
 } ArvGvStreamFrameData;
@@ -101,14 +105,17 @@ typedef struct {
 	guint n_timeouts;
 	guint n_underruns;
 	guint n_aborteds;
+	guint n_missing_frames;
+
 	guint n_size_mismatch_errors;
 
-	guint n_late_packets;
+	guint n_received_packets;
+	guint n_missing_packets;
+	guint n_error_packets;
+	guint n_ignored_packets;
 	guint n_resend_requests;
 	guint n_resent_packets;
-	guint n_missing_packets;
 	guint n_duplicated_packets;
-	guint n_missing_frames;
 
 	ArvStatistic *statistic;
 	guint32 statistic_count;
@@ -130,10 +137,10 @@ _send_packet_request (ArvGvStreamThreadData *thread_data,
 	packet = arv_gvcp_packet_new_packet_resend_cmd (frame_id, first_block, last_block,
 							thread_data->packet_count++, &packet_size);
 
-	arv_debug ("stream-thread", "[GvStream::send_packet_request] frame_id = %u (%d - %d)",
-		   frame_id, first_block, last_block);
+	arv_log_stream_thread ("[GvStream::send_packet_request] frame_id = %u (%d - %d)",
+			       frame_id, first_block, last_block);
 
-	arv_gvcp_packet_debug (packet);
+	arv_gvcp_packet_debug (packet, ARV_DEBUG_LEVEL_LOG);
 
 	g_socket_send_to (thread_data->socket, thread_data->device_address, (const char *) packet, packet_size,
 			  NULL, NULL);
@@ -167,8 +174,7 @@ _update_socket (ArvGvStreamThreadData *thread_data, ArvBuffer *buffer)
 	if (buffer_size != thread_data->current_socket_buffer_size) {
 		setsockopt (fd, SOL_SOCKET, SO_RCVBUF, &buffer_size, sizeof (buffer_size));
 		thread_data->current_socket_buffer_size = buffer_size;
-		arv_debug ("stream-thread", "[GvStream::update_socket] Socket buffer size set to %d",
-			 buffer_size);
+		arv_debug_stream_thread ("[GvStream::update_socket] Socket buffer size set to %d", buffer_size);
 	}
 }
 
@@ -198,8 +204,8 @@ _process_data_leader (ArvGvStreamThreadData *thread_data,
 
 	if (frame->packet_data[packet_id].time_us > 0) {
 		thread_data->n_resent_packets++;
-		arv_debug ("stream-thread", "[GvStream::_process_data_leader] Received resent packet %u for frame %u",
-			   packet_id, frame->frame_id);
+		arv_log_stream_thread ("[GvStream::_process_data_leader] Received resent packet %u for frame %u",
+				       packet_id, frame->frame_id);
 	}
 }
 
@@ -217,8 +223,8 @@ _process_data_block (ArvGvStreamThreadData *thread_data,
 	if (frame->buffer->status != ARV_BUFFER_STATUS_FILLING)
 		return;
 
-	if (packet_id > frame->n_packets - 2) {
-		arv_gvsp_packet_debug (packet, read_count);
+	if (packet_id > frame->n_packets - 2 || packet_id < 1) {
+		arv_gvsp_packet_debug (packet, read_count, ARV_DEBUG_LEVEL_DEBUG);
 		frame->buffer->status = ARV_BUFFER_STATUS_WRONG_PACKET_ID;
 		return;
 	}
@@ -228,10 +234,10 @@ _process_data_block (ArvGvStreamThreadData *thread_data,
 	block_end = block_size + block_offset;
 
 	if (block_end > frame->buffer->size) {
-		arv_debug ("stream-thread", "[GvStream::_process_data_block] %d unexpected bytes in packet %u"
-			   " for frame %u",
-			   block_end, frame->buffer->size,
-			   packet_id, frame->frame_id);
+		arv_debug_stream_thread ("[GvStream::_process_data_block] %d unexpected bytes in packet %u "
+					 " for frame %u",
+					 block_end - frame->buffer->size,
+					 packet_id, frame->frame_id);
 		thread_data->n_size_mismatch_errors++;
 
 		block_end = frame->buffer->size;
@@ -242,8 +248,8 @@ _process_data_block (ArvGvStreamThreadData *thread_data,
 
 	if (frame->packet_data[packet_id].time_us > 0) {
 		thread_data->n_resent_packets++;
-		arv_debug ("stream-thread", "[GvStream::_process_data_block] Received resent packet %u for frame %u",
-			   packet_id, frame->frame_id);
+		arv_log_stream_thread ("[GvStream::_process_data_block] Received resent packet %u for frame %u",
+				       packet_id, frame->frame_id);
 	}
 }
 
@@ -263,8 +269,8 @@ _process_data_trailer (ArvGvStreamThreadData *thread_data,
 
 	if (frame->packet_data[packet_id].time_us > 0) {
 		thread_data->n_resent_packets++;
-		arv_debug ("stream-thread", "[GvStream::_process_data_trailer] Received resent packet %u for frame %u",
-			   packet_id, frame->frame_id);
+		arv_log_stream_thread ("[GvStream::_process_data_trailer] Received resent packet %u for frame %u",
+				       packet_id, frame->frame_id);
 	}
 }
 
@@ -280,6 +286,7 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
 	ArvBuffer *buffer;
 	GSList *iter;
 	guint n_packets = 0;
+	gint16 frame_id_inc;
 
 	for (iter = thread_data->frames; iter != NULL; iter = iter->next) {
 		frame = iter->data;
@@ -289,6 +296,13 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
 		}
 	}
 
+	frame_id_inc = (gint16) frame_id - (gint16) thread_data->last_frame_id;
+	if (frame_id_inc < 1  && frame_id_inc > -ARV_GV_STREAM_DISCARD_LATE_FRAME_THRESHOLD) {
+		arv_debug_stream_thread ("[GvStream::_find_frame_data] Discard late frame %u (last: %u)",
+					 frame_id, thread_data->last_frame_id);
+		return NULL;
+	}
+
 	buffer = arv_stream_pop_input_buffer (thread_data->stream);
 	if (buffer == NULL) {
 		thread_data->n_underruns++;
@@ -298,6 +312,8 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
 
 	frame = g_new0 (ArvGvStreamFrameData, 1);
 
+	frame->error_packet_received = FALSE;
+
 	frame->frame_id = frame_id;
 	frame->last_valid_packet = -1;
 
@@ -318,24 +334,17 @@ _find_frame_data (ArvGvStreamThreadData *thread_data,
 				       ARV_STREAM_CALLBACK_TYPE_START_BUFFER,
 				       NULL);
 
-	{
-		gint32 frame_id_inc;
+	thread_data->last_frame_id = frame_id;
 
-		frame_id_inc = (gint )frame_id - (gint) thread_data->last_frame_id;
-		if (frame_id_inc > 0) {
-			thread_data->last_frame_id = frame_id;
-			if (frame_id_inc != 1) {
-				thread_data->n_missing_frames++;
-				arv_debug ("stream-thread", "[GvStream::_find_frame_data]"
-					   " Missed %d frame(s) before %u",
-					   frame_id_inc - 1, frame_id);
-			}
-		}
+	if (frame_id_inc > 1) {
+		thread_data->n_missing_frames++;
+		arv_log_stream_thread ("[GvStream::_find_frame_data] Missed %d frame(s) before %u",
+				       frame_id_inc - 1, frame_id);
 	}
 
 	thread_data->frames = g_slist_append (thread_data->frames, frame);
 
-	arv_debug ("stream-thread", "[GvStream::_find_frame_data] Start frame %u", frame_id);
+	arv_log_stream_thread ("[GvStream::_find_frame_data] Start frame %u", frame_id);
 
 	return frame;
 }
@@ -348,7 +357,8 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 {
 	int i;
 
-	if (thread_data->packet_resend == ARV_GV_STREAM_PACKET_RESEND_NEVER)
+	if (thread_data->packet_resend == ARV_GV_STREAM_PACKET_RESEND_NEVER ||
+	    frame->error_packet_received)
 		return;
 
 	if (packet_id < frame->n_packets) {
@@ -364,10 +374,10 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 				if (first_missing >= 0) {
 					int j;
 
-					arv_debug ("stream-thread", "[GvStream::_missing_packet_check]"
-						   " Resend request at dt = %Lu, packet id = %u/%u",
-						   time_us - frame->first_packet_time_us,
-						   packet_id, frame->n_packets);
+					arv_log_stream_thread ("[GvStream::_missing_packet_check]"
+							       " Resend request at dt = %Lu, packet id = %u/%u",
+							       time_us - frame->first_packet_time_us,
+							       packet_id, frame->n_packets);
 
 					_send_packet_request (thread_data, frame->frame_id,
 							      first_missing, i - 1);
@@ -382,10 +392,10 @@ _missing_packet_check (ArvGvStreamThreadData *thread_data,
 		if (first_missing >= 0) {
 			int j;
 
-			arv_debug ("stream-thread", "[GvStream::_missing_packet_check]"
-				   " Resend request at dt = %Lu, packet id = %u/%u",
-				   time_us - frame->first_packet_time_us,
-				   packet_id, frame->n_packets);
+			arv_log_stream_thread ("[GvStream::_missing_packet_check]"
+					       " Resend request at dt = %Lu, packet id = %u/%u",
+					       time_us - frame->first_packet_time_us,
+					       packet_id, frame->n_packets);
 
 			_send_packet_request (thread_data, frame->frame_id,
 					      first_missing, i - 1);
@@ -426,7 +436,7 @@ _close_frame (ArvGvStreamThreadData *thread_data, ArvGvStreamFrameData *frame)
 	g_get_current_time (&current_time);
 	current_time_us = current_time.tv_sec * 1000000 + current_time.tv_usec;
 	if (thread_data->statistic_count > 5) {
-		arv_statistic_fill (thread_data->statistic, 1,
+		arv_statistic_fill (thread_data->statistic, 0,
 				    current_time_us - frame->first_packet_time_us,
 				    frame->buffer->frame_id);
 	} else
@@ -434,7 +444,7 @@ _close_frame (ArvGvStreamThreadData *thread_data, ArvGvStreamFrameData *frame)
 
 	arv_stream_push_output_buffer (thread_data->stream, frame->buffer);
 
-	arv_debug ("stream-thread", "[GvStream::_close_frame] Close frame %u", frame->frame_id);
+	arv_log_stream_thread ("[GvStream::_close_frame] Close frame %u", frame->frame_id);
 
 	frame->buffer = NULL;
 	frame->frame_id = 0;
@@ -459,8 +469,8 @@ _check_frame_completion (ArvGvStreamThreadData *thread_data,
 		    thread_data->packet_resend == ARV_GV_STREAM_PACKET_RESEND_NEVER &&
 		    iter->next != NULL) {
 			frame->buffer->status = ARV_BUFFER_STATUS_MISSING_PACKETS;
-			arv_debug ("stream-thread", "[GvStream::_check_frame_completion] Incomplete frame %u",
-				   frame->frame_id);
+			arv_debug_stream_thread ("[GvStream::_check_frame_completion] Incomplete frame %u",
+						 frame->frame_id);
 			_close_frame (thread_data, frame);
 			thread_data->frames = iter->next;
 			g_slist_free_1 (iter);
@@ -471,8 +481,8 @@ _check_frame_completion (ArvGvStreamThreadData *thread_data,
 		if (can_close_frame &&
 		    frame->last_valid_packet == frame->n_packets - 1) {
 			frame->buffer->status = ARV_BUFFER_STATUS_SUCCESS;
-			arv_debug ("stream-thread", "[GvStream::_check_frame_completion] Completed frame %u",
-				   frame->frame_id);
+			arv_log_stream_thread ("[GvStream::_check_frame_completion] Completed frame %u",
+					       frame->frame_id);
 			_close_frame (thread_data, frame);
 			thread_data->frames = iter->next;
 			g_slist_free_1 (iter);
@@ -483,8 +493,22 @@ _check_frame_completion (ArvGvStreamThreadData *thread_data,
 		if (can_close_frame &&
 		    time_us - frame->last_packet_time_us >= thread_data->frame_retention_us) {
 			frame->buffer->status = ARV_BUFFER_STATUS_TIMEOUT;
-			arv_debug ("stream-thread", "[GvStream::_check_frame_completion] Timeout for frame %u",
-				   frame->frame_id);
+			arv_debug_stream_thread ("[GvStream::_check_frame_completion] Timeout for frame %u "
+						 "at dt = %Lu",
+						 frame->frame_id,
+						 time_us - frame->first_packet_time_us);
+#if 0
+			if (arv_debug_check (&arv_debug_category_stream_thread, ARV_DEBUG_LEVEL_LOG)) {
+				int i;
+				arv_log_stream_thread ("frame_id          = %Lu", frame->frame_id);
+				arv_log_stream_thread ("last_valid_packet = %d", frame->last_valid_packet);
+				for (i = 0; i < frame->n_packets; i++) {
+					arv_log_stream_thread ("%d - time = %Lu%s", i,
+							       frame->packet_data[i].time_us,
+							       frame->packet_data[i].received ? " - OK" : "");
+				}
+			}
+#endif
 			_close_frame (thread_data, frame);
 			thread_data->frames = iter->next;
 			g_slist_free_1 (iter);
@@ -536,9 +560,15 @@ arv_gv_stream_thread (void *data)
 	int timeout_ms;
 	int n_events;
 	int i;
+	gboolean first_packet = TRUE;
 
 	thread_data->frames = NULL;
 
+	arv_debug_stream_thread ("[GvStream::stream_thread] Packet timeout = %g ms",
+				 thread_data->packet_timeout_us / 1000.0);
+	arv_debug_stream_thread ("[GvStream::stream_thread] Frame retention = %g ms",
+				 thread_data->frame_retention_us / 1000.0);
+
 	if (thread_data->callback != NULL)
 		thread_data->callback (thread_data->user_data, ARV_STREAM_CALLBACK_TYPE_INIT, NULL);
 
@@ -560,6 +590,7 @@ arv_gv_stream_thread (void *data)
 		time_us = current_time.tv_sec * 1000000 + current_time.tv_usec;
 
 		if (n_events > 0) {
+			thread_data->n_received_packets++;
 
 			read_count = g_socket_receive (thread_data->socket, (char *) packet,
 						       ARV_GV_STREAM_INCOMING_BUFFER_SIZE, NULL, NULL);
@@ -567,38 +598,60 @@ arv_gv_stream_thread (void *data)
 			frame_id = arv_gvsp_packet_get_frame_id (packet);
 			packet_id = arv_gvsp_packet_get_packet_id (packet);
 
+			if (first_packet) {
+				thread_data->last_frame_id = frame_id - 1;
+				first_packet = FALSE;
+			}
+
 			frame = _find_frame_data (thread_data, frame_id, packet, packet_id, read_count, time_us);
 
 			if (frame != NULL) {
 
-				/* Check for duplicated packets */
-				if (packet_id < frame->n_packets) {
-					if (frame->packet_data[packet_id].received)
-						thread_data->n_duplicated_packets++;
-					else
-						frame->packet_data[packet_id].received = TRUE;
+				if (arv_gvsp_packet_get_packet_type (packet) != ARV_GVSP_PACKET_TYPE_OK) {
+					arv_debug_stream_thread ("[GvStream::stream_thread]"
+								 " Error packet at dt = %Lu, packet id = %u"
+								 " frame id = %u",
+								 time_us - frame->first_packet_time_us,
+								 packet_id, frame->frame_id);
+					arv_gvsp_packet_debug (packet, read_count, ARV_DEBUG_LEVEL_DEBUG);
+					frame->error_packet_received = TRUE;
+
+					thread_data->n_error_packets++;
+				} else {
+					/* Check for duplicated packets */
+					if (packet_id < frame->n_packets) {
+						if (frame->packet_data[packet_id].received)
+							thread_data->n_duplicated_packets++;
+						else
+							frame->packet_data[packet_id].received = TRUE;
+					}
+
+					/* Keep track of last packet of a continuous block starting from packet 0 */
+					for (i = frame->last_valid_packet + 1; i < frame->n_packets; i++)
+						if (!frame->packet_data[i].received)
+							break;
+					frame->last_valid_packet = i - 1;
+
+					switch (arv_gvsp_packet_get_content_type (packet)) {
+						case ARV_GVSP_CONTENT_TYPE_DATA_LEADER:
+							_process_data_leader (thread_data, frame, packet, packet_id);
+							break;
+						case ARV_GVSP_CONTENT_TYPE_DATA_BLOCK:
+							_process_data_block (thread_data, frame, packet, packet_id,
+									     read_count);
+							break;
+						case ARV_GVSP_CONTENT_TYPE_DATA_TRAILER:
+							_process_data_trailer (thread_data, frame, packet, packet_id);
+							break;
+						default:
+							thread_data->n_ignored_packets++;
+							break;
+					}
+
+					_missing_packet_check (thread_data, frame, packet_id, time_us);
 				}
-
-				/* Keep track of last packet of a continuous block starting from packet 0 */
-				for (i = frame->last_valid_packet + 1; i < frame->n_packets; i++)
-					if (!frame->packet_data[i].received)
-						break;
-				frame->last_valid_packet = i - 1;
-
-				switch (arv_gvsp_packet_get_packet_type (packet)) {
-					case ARV_GVSP_PACKET_TYPE_DATA_LEADER:
-						_process_data_leader (thread_data, frame, packet, packet_id);
-						break;
-					case ARV_GVSP_PACKET_TYPE_DATA_BLOCK:
-						_process_data_block (thread_data, frame, packet, packet_id, read_count);
-						break;
-					case ARV_GVSP_PACKET_TYPE_DATA_TRAILER:
-						_process_data_trailer (thread_data, frame, packet, packet_id);
-						break;
-				}
-
-				_missing_packet_check (thread_data, frame, packet_id, time_us);
-			}
+			} else
+				thread_data->n_ignored_packets++;
 		} else
 			frame = NULL;
 
@@ -687,21 +740,24 @@ arv_gv_stream_new (GInetAddress *device_address, guint16 port,
 	thread_data->n_completed_buffers = 0;
 	thread_data->n_failures = 0;
 	thread_data->n_underruns = 0;
+	thread_data->n_aborteds = 0;
+	thread_data->n_timeouts = 0;
+	thread_data->n_missing_frames = 0;
+
 	thread_data->n_size_mismatch_errors = 0;
+
+	thread_data->n_received_packets = 0;
 	thread_data->n_missing_packets = 0;
-	thread_data->n_late_packets = 0;
+	thread_data->n_error_packets = 0;
+	thread_data->n_ignored_packets = 0;
 	thread_data->n_resent_packets = 0;
 	thread_data->n_resend_requests = 0;
 	thread_data->n_duplicated_packets = 0;
-	thread_data->n_missing_frames = 0;
-	thread_data->n_aborteds = 0;
-	thread_data->n_timeouts = 0;
 
-	thread_data->statistic = arv_statistic_new (2, 5000, 200, 0);
+	thread_data->statistic = arv_statistic_new (1, 5000, 200, 0);
 	thread_data->statistic_count = 0;
 
-	arv_statistic_set_name (thread_data->statistic, 0, "Timestamp delta");
-	arv_statistic_set_name (thread_data->statistic, 1, "Buffer reception time");
+	arv_statistic_set_name (thread_data->statistic, 0, "Buffer reception time");
 
 	thread_data->socket_buffer_option = ARV_GV_STREAM_SOCKET_BUFFER_FIXED;
 	thread_data->socket_buffer_size = 0;
@@ -834,34 +890,41 @@ arv_gv_stream_finalize (GObject *object)
 		g_object_unref (thread_data->device_address);
 
 		statistic_string = arv_statistic_to_string (thread_data->statistic);
-		arv_debug ("stream", statistic_string);
+		arv_debug_stream (statistic_string);
 		g_free (statistic_string);
 		arv_statistic_free (thread_data->statistic);
 
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_completed_buffers    = %d", thread_data->n_completed_buffers);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_failures             = %d", thread_data->n_failures);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_timeouts             = %d", thread_data->n_timeouts);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_aborteds             = %d", thread_data->n_aborteds);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_underruns            = %d", thread_data->n_underruns);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_size_mismatch_errors = %d", thread_data->n_size_mismatch_errors);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_missing_packets      = %d", thread_data->n_missing_packets);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_late_packets         = %d", thread_data->n_late_packets);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_resend_requests      = %d", thread_data->n_resend_requests);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_resent_packets       = %d", thread_data->n_resent_packets);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_duplicated_packets   = %d", thread_data->n_duplicated_packets);
-		arv_debug ("stream",
-			   "[GvStream::finalize] n_missing_frames       = %d", thread_data->n_missing_frames);
+		arv_debug_stream ("[GvStream::finalize] n_completed_buffers    = %u",
+				  thread_data->n_completed_buffers);
+		arv_debug_stream ("[GvStream::finalize] n_failures             = %u",
+				  thread_data->n_failures);
+		arv_debug_stream ("[GvStream::finalize] n_timeouts             = %u",
+				  thread_data->n_timeouts);
+		arv_debug_stream ("[GvStream::finalize] n_aborteds             = %u",
+				  thread_data->n_aborteds);
+		arv_debug_stream ("[GvStream::finalize] n_underruns            = %u",
+				  thread_data->n_underruns);
+		arv_debug_stream ("[GvStream::finalize] n_missing_frames       = %u",
+				  thread_data->n_missing_frames);
+
+		arv_debug_stream ("[GvStream::finalize] n_size_mismatch_errors = %u",
+				  thread_data->n_size_mismatch_errors);
+
+		arv_debug_stream ("[GvStream::finalize] n_received_packets     = %u",
+				  thread_data->n_received_packets);
+		arv_debug_stream ("[GvStream::finalize] n_missing_packets      = %u",
+				  thread_data->n_missing_packets);
+		arv_debug_stream ("[GvStream::finalize] n_error_packets        = %u",
+				  thread_data->n_error_packets);
+		arv_debug_stream ("[GvStream::finalize] n_ignored_packets      = %u",
+				  thread_data->n_ignored_packets);
+
+		arv_debug_stream ("[GvStream::finalize] n_resend_requests      = %u",
+				  thread_data->n_resend_requests);
+		arv_debug_stream ("[GvStream::finalize] n_resent_packets       = %u",
+				  thread_data->n_resent_packets);
+		arv_debug_stream ("[GvStream::finalize] n_duplicated_packets   = %u",
+				  thread_data->n_duplicated_packets);
 
 		g_free (thread_data);
 
